unit Test;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, VclTee.TeeGDIPlus, Vcl.StdCtrls,
  VCLTee.TeEngine, VCLTee.TeeTools, VCLTee.Series, Vcl.ExtCtrls,
  VCLTee.TeeProcs, VCLTee.Chart, VCLTee.TeeComma, VCLTee.DBChart,Vcl.Grids,
  Vcl.DBGrids, Data.DB, Datasnap.DBClient;

type
  TForm2 = class(TForm)
    Panel1: TPanel;
    ScrollBar1: TScrollBar;
    TeeCommander1: TTeeCommander;
    DBChart: TDBChart;
    Series1: TLineSeries;
    Series3: TLineSeries;
    ClientDataSet: TClientDataSet;
    ClientDataSetNonCompDate: TDateField;
    ClientDataSetNonCompTot: TBCDField;
    dsClientData: TDataSource;
    Memo1: TMemo;
    Button1: TButton;
    Button2: TButton;
    Label1: TLabel;
    Label2: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure ScrollBar1Change(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure DBChartClickSeries(Sender: TCustomChart; Series: TChartSeries;
      ValueIndex: Integer; Button: TMouseButton; Shift: TShiftState; X,
      Y: Integer);
    procedure DBChartMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
  private
    { Private declarations }
    nPointsToShow, iSegmentsPerDataPoint: Integer;
    function InterpolateLineSeries(Series: TChartSeries;XValue: Double): Double; overload;
    function InterpolateLineSeries(Series: TChartSeries; FirstIndex,
                                  LastIndex: Integer; XValue: Double): Double; overload;
  public
    { Public declarations }
  end;

var
  Form2: TForm2;

implementation

{$R *.dfm}

function TForm2.InterpolateLineSeries(Series: TChartSeries;
  XValue: Double): Double;
begin
  result:=InterpolateLineSeries(Series,Series.FirstDisplayedIndex,Series.LastValueIndex,XValue);
end;

function TForm2.InterpolateLineSeries(Series: TChartSeries; FirstIndex,
  LastIndex: Integer; XValue: Double): Double;
var
  Index: Integer;
  dx,dy: Double;
begin
  for Index:=FirstIndex to LastIndex do
    if Series.XValues.Value[Index]>XValue then break;

  //safeguard
  if (Index<1) then Index:=1
  else if (Index>=Series.Count) then Index:=Series.Count-1;

  // y=(y2-y1)/(x2-x1)*(x-x1)+y1
  dx:=Series.XValues.Value[Index] - Series.XValues.Value[Index-1];
  dy:=Series.YValues.Value[Index] - Series.YValues.Value[Index-1];

  if (dx<>0) then
    result:=dy*(XValue - Series.XValues.Value[Index-1])/dx + Series.YValues.Value[Index-1]
  else result:=0;
end;

procedure TForm2.Button1Click(Sender: TObject);
var Min, Max: double;
begin
  ScrollBar1.Enabled := True;

  DBChart[1].Active := False;
  DBChart[0].Active := True;

  ScrollBar1.Max := DBChart[0].Count-nPointsToShow-1;
  ScrollBar1.Position := 0;

  Min := DBChart[0].XValue[ScrollBar1.Position];
  Max := DBChart[0].XValue[ScrollBar1.Position+nPointsToShow];
  DBChart.Axes.Bottom.SetMinMax(Min, Max);

  iSegmentsPerDataPoint := Round((DBChart.BottomAxis.IAxisSize /
    (((DBChart.BottomAxis.Maximum)-(DBChart.BottomAxis.Minimum)+1)-1))/4);
  // 4 is the number of pixels to move with each Scrollbar Position increment.

  ScrollBar1Change(Self);
end;

procedure TForm2.Button2Click(Sender: TObject);
var Min, Max: double;
begin
  ScrollBar1.Enabled := True;

  DBChart[0].Active := False;
  DBChart[1].Active := True;

  ScrollBar1.Position := 0;
  ScrollBar1.Max := DBChart[1].Count-nPointsToShow-1;

  Min := DBChart[1].XValue[ScrollBar1.Position];
  Max := DBChart[1].XValue[ScrollBar1.Position+nPointsToShow];
  DBChart.Axes.Bottom.SetMinMax(Min, Max);

  iSegmentsPerDataPoint := Round((DBChart.BottomAxis.IAxisSize /
    (((DBChart.BottomAxis.Maximum)-(DBChart.BottomAxis.Minimum)+1)-1))/4);
  // 4 is the number of pixels to move with each Scrollbar Position increment.

// This should not be set in this way until we have SmoothScroll working for Series2.
//  ScrollBar1.Max := (DBChart.GetMaxValuesCount-nPointsToShow) * iSegmentsPerDataPoint;

  //ScrollBar1.Position := 0;

  ScrollBar1Change(Self);
end;

procedure TForm2.DBChartClickSeries(Sender: TCustomChart; Series: TChartSeries;
  ValueIndex: Integer; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var xval:double;
begin
xval := DBChart.Axes.Bottom.CalcPosPoint(X);
showmessage(
'Caclpospoint: '+DateToStr(FloatToDateTime(xval))+chr(13)+
'Series.ValueIndex: '+IntToStr(ValueIndex)+chr(13)+
'Series1.XValueToText: '+ Series1.XValueToText(Series1.XValue[ValueIndex])+chr(13)+
'XValue: '+FloatToStr(DBChart[Series1.SeriesIndex].XValues[ValueIndex])+chr(13)+
'1-Date: '+DBChart[Series1.SeriesIndex].XLabel[ValueIndex]+chr(13)+
'2-Date: '+DBChart[Series1.SeriesIndex].XValueToText(ValueIndex)+chr(13)+
'3-Date: '+DBChart[Series1.SeriesIndex].XValueToText(DBChart[Series1.SeriesIndex].XValue[ValueIndex])+chr(13)+
'4-Date: '+FloatToStr(DBChart[Series1.SeriesIndex].XValue[ValueIndex])+chr(13)+
'5-Date: '+Floattostr(DBChart[Series1.SeriesIndex].XValues.Items[ValueIndex])
);
end;

procedure TForm2.DBChartMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
var xval:double;
begin
  xval := DBChart.Axes.Bottom.CalcPosPoint(X);

  label1.Caption := DateToStr(FloatToDateTime(xval));
  label2.Caption := FloatToStrF(InterpolateLineSeries(DBChart[1],xval), ffNumber, 8, 2)
end;

procedure TForm2.FormCreate(Sender: TObject);
var i: integer;
begin
  DBChart.Axes.Bottom.Automatic:=True;
  DBChart[0].Active:=True;

  DBChart[0].Color := clWebLightSteelBlue;
  DBChart[0].AddXY(EncodeDate(2004, 1, 4), 7, '01/04/2004');
  DBChart[0].AddXY(EncodeDate(2004, 1, 11), 17,'01/11/2004');
  DBChart[0].AddXY(EncodeDate(2004, 1, 18), 6, '01/18/2004');
  DBChart[0].AddXY(EncodeDate(2004, 1, 25), 12,'01/25/2004');
  DBChart[0].AddXY(EncodeDate(2004, 2, 1), 5, '02/01/2004');
  DBChart[0].AddXY(EncodeDate(2004, 2, 8), 7, '02/08/2004');
  DBChart[0].AddXY(EncodeDate(2004, 2, 15), 14,'02/15/2004');
  DBChart[0].AddXY(EncodeDate(2004, 2, 22), 4, '02/22/2004');
  DBChart[0].AddXY(EncodeDate(2004, 3, 1), 12,'03/01/2004');
  DBChart[0].AddXY(EncodeDate(2004, 3, 8), 9, '03/08/2004');
  DBChart[0].AddXY(EncodeDate(2004, 3, 15), 17,'03/15/2004');
  DBChart[0].AddXY(EncodeDate(2004, 3, 22), 2, '03/22/2004');
  DBChart[0].AddXY(EncodeDate(2004, 3, 29), 5, '03/29/2004');
  DBChart[0].AddXY(EncodeDate(2004, 4, 6), 10,'04/06/2004');
  DBChart[0].AddXY(EncodeDate(2004, 4, 13), 6, '04/13/2004');
  DBChart[0].AddXY(EncodeDate(2004, 4, 20), 17,'04/20/2004');
  DBChart[0].AddXY(EncodeDate(2004, 4, 27), 27,'04/27/2004');
  DBChart[0].AddXY(EncodeDate(2004, 5, 4), 16,'05/04/2004');
  DBChart[0].AddXY(EncodeDate(2004, 5, 11), 22,'05/11/2004');
  DBChart[0].AddXY(EncodeDate(2004, 5, 18), 15,'05/18/2004');
  DBChart[0].AddXY(EncodeDate(2004, 5, 25), 17,'05/25/2004');
  DBChart[0].AddXY(EncodeDate(2004, 6, 1), 24,'06/01/2004');
  DBChart[0].AddXY(EncodeDate(2004, 6, 8), 14,'06/08/2004');
  DBChart[0].AddXY(EncodeDate(2004, 6, 15), 22,'06/15/2004');
  DBChart[0].AddXY(EncodeDate(2004, 6, 22), 19,'06/22/2004');
  DBChart[0].AddXY(EncodeDate(2004, 6, 29), 27,'06/29/2004');
  DBChart[0].AddXY(EncodeDate(2004, 7, 6), 12,'07/06/2004');
  DBChart[0].AddXY(EncodeDate(2004, 7, 13), 15,'07/13/2004');
  DBChart[0].AddXY(EncodeDate(2004, 7, 20), 20,'07/20/2004');
  DBChart[0].AddXY(EncodeDate(2004, 7, 27), 16,'07/27/2004');
  DBChart[0].AddXY(EncodeDate(2004, 7, 3), 27,'07/03/2004');
  DBChart[0].AddXY(EncodeDate(2004, 7, 10), 37,'07/10/2004');
  DBChart[0].AddXY(EncodeDate(2004, 7, 17), 26,'07/17/2004');
  DBChart[0].AddXY(EncodeDate(2004, 7, 24), 32,'07/24/2004');
  DBChart[0].AddXY(EncodeDate(2004, 8, 3), 25,'08/03/2004');
  DBChart[0].AddXY(EncodeDate(2004, 8, 10), 27,'08/10/2004');
  DBChart[0].AddXY(EncodeDate(2004, 8, 17), 34,'08/17/2004');
  DBChart[0].AddXY(EncodeDate(2004, 8, 24), 24,'08/24/2004');
  DBChart[0].AddXY(EncodeDate(2004, 8, 31), 32,'08/31/2004');
  DBChart[0].AddXY(EncodeDate(2004, 9, 7), 29,'09/07/2004');
  DBChart[0].AddXY(EncodeDate(2004, 9, 14), 37,'09/14/2004');
  DBChart[0].AddXY(EncodeDate(2004, 9, 21), 22,'09/21/2004');
  DBChart[0].AddXY(EncodeDate(2004, 9, 28), 25,'09/28/2004');
  DBChart[0].AddXY(EncodeDate(2004, 10, 5), 30,'10/05/2004');
  DBChart[0].AddXY(EncodeDate(2004, 10, 12), 26,'10/12/2004');

  nPointsToShow := 10;

  ScrollBar1.Min := 0;
  ScrollBar1.Max := DBChart[0].Count-nPointsToShow-1;

  for I := 0 to DBChart[1].Count - 1 do
  begin
    DBChart[1].Labels[i]:=DateToStr(DBChart[1].XValues[i]);
    DBChart[1].XValues[i]:=i;
  end;

end;

procedure TForm2.ScrollBar1Change(Sender: TObject);
var Min, Max: double;
begin

  if DBChart[0].Active then
    begin
      Min := DBChart[0].XValue[ScrollBar1.Position];
      Max := DBChart[0].XValue[ScrollBar1.Position+nPointsToShow];
    end;

  if DBChart[1].Active then
    begin
      Min := DBChart[1].XValue[ScrollBar1.Position];
      Max := DBChart[1].XValue[ScrollBar1.Position+nPointsToShow];

// Comment out these next 2 lines.  It seems it should work
// just like DBChart[0] but it doesn't - what am I missing?

      //DBChart.Axes.Bottom.SetMinMax(Min, Max);
      //Exit;
    end;

//  iSegmentsPerDataPoint := Round((DBChart.BottomAxis.IAxisSize /
//    (((DBChart.BottomAxis.Maximum)-(DBChart.BottomAxis.Minimum)+1)-1))/4);
  iSegmentsPerDataPoint := Round((DBChart.BottomAxis.IAxisSize /
    (((DBChart.BottomAxis.Maximum)-(DBChart.BottomAxis.Minimum)+1)-1))/4);
  // 4 is the number of pixels to move with each Scrollbar Position increment.

  // Make sure ScrollBar.Max is set correctly.
  ScrollBar1.Max := (DBChart.GetMaxValuesCount-nPointsToShow) * iSegmentsPerDataPoint;

  // Scroll the chart by setting BottomAxis Min & Max.
  DBChart.Axes.Bottom.SetMinMax(
    (ScrollBar1.Position / iSegmentsPerDataPoint),
    ((ScrollBar1.Position / iSegmentsPerDataPoint) + nPointsToShow - 1) );
end;

end.
